/* Using a hash object to accomodate dynamic lags:              */
/*  CURRSEQ merely provides a running count of records for      */
/*     each PRODUCT                                             */
/*  LAGS holds the history of values for MONTH and SALES for    */
/*     each PRODUCT.                                            */

data dynamic_lags (drop=_:);
  set sales;
  _seq=0;

  if _n_=1 then do;
    /* Hash object CURRSEQ: track current count by product     */
    declare hash currseq();
      currseq.definekey('product');
      currseq.definedata('_seq');
      currseq.definedone();

    _sl=. ;  _ml=. ;                 /* Lagged Sales and Month */
    /* hash object LAGS: _SL and _ML keyed by PRODUCT and _SEQ */
    declare hash lags(); 
      lags.definekey('product','_seq');
      lags.definedata('_sl','_ml');
      lags.definedone();
  end;

  _rc=currseq.find(); /* Retrieve _SEQ for current PRODUCT      */
  _seq=_seq+1;
  currseq.replace();  /* Update current product's running count */

  /* Add SALES (as data element _SL) and MONTH (as _ML) to LAGS */
  /* object, keyed on PRODUCT and _SEQ                          */
  lags.add(key:product,key:_seq,data:sales,data:month);

  _rc=lags.find(key:product,key:_seq-1);
  if _rc=0 then change_rate=(sales-_sl)/(month-_ml);

run;
